/*
 * file CqQueryFolder.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqQueryFolder
 *
 * (C) Copyright IBM Corporation 2004, 2009.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import java.util.EnumSet;
import java.util.List;

import javax.wvcm.Activity;
import javax.wvcm.ControllableResource;
import javax.wvcm.Feedback;
import javax.wvcm.Folder;
import javax.wvcm.Resource;
import javax.wvcm.ResourceList;
import javax.wvcm.Version;
import javax.wvcm.Workspace;
import javax.wvcm.WvcmException;
import javax.wvcm.Folder.RebindFlag;
import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.WvcmException.ReasonCode;

import com.ibm.rational.wvcm.stp.StpAccessControlEntry;
import com.ibm.rational.wvcm.stp.StpAccessControlledResource;
import com.ibm.rational.wvcm.stp.StpAccessControlEntry.EntryType;
import com.ibm.rational.wvcm.stpex.StpExBase;


/**
 * A proxy for a query folder.
 * <p>
 * The user-friendly specification for the location of a query folder has the
 * form
 * 
 * <pre>
 *  <b>cq.query:</b><i>&lt;parent-folder-path&gt;</i>/<i>&lt;folder-name&gt;</i>@<i>&lt;db-set&gt;</i>/<i>&lt;user-db&gt;</i>
 * </pre>
 */
public interface CqQueryFolder extends Folder, CqQueryFolderItem,
                StpAccessControlledResource
{
    /**
     * Creates a new folder at the location specified in this proxy.
     * 
     * @param feedback A Feedback object specifying the information desired from
     *            the new folder after it has been created.
     * @return A proxy for the newly-created folder populated with the requested
     *         properties. The creation of the new folder will have been
     *         delivered to the database before the method returns.
     * @throws WvcmException If the folder cannot be created at the specified
     *             location.
     */
    CqQueryFolder doCreateFolder(Feedback feedback) throws WvcmException;

    /**
     * Creates a new folder at the location specified in this proxy.
     * 
     * @param feedback A Feedback object specifying the information desired from
     *            the new folder after it has been created.
     * @param deliveryOrder A specification of what resources are to be
     *            delivered to the database after the folder has been created.
     * @return A proxy for the newly-created folder populated with the requested
     *         properties.
     * @throws WvcmException If the folder cannot be created at the specified
     *             location or if any of the resources to be delivered to the
     *             database cannot be delivered.
     */
    CqQueryFolder doCreateFolder(Feedback feedback,
                                 List<CqContextResource> deliveryOrder)
        throws WvcmException;

    /**
     * The queries, query folders and other items that are immediately contained
     * within this folder.
     */
    PropertyName<ResourceList<CqQueryFolderItem>> QUERY_FOLDER_ITEMS =
        new PropertyName<ResourceList<CqQueryFolderItem>>(PROPERTY_NAMESPACE,
                                                          "query-folder-items");

    /**
     * Returns the value of the {@link #QUERY_FOLDER_ITEMS QUERY_FOLDER_ITEMS}
     * property as defined by this proxy.
     * 
     * @return A ResourceList containing a proxy for each immediate member of
     *         this query folder.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #QUERY_FOLDER_ITEMS QUERY_FOLDER_ITEMS} property.
     */
    ResourceList<CqQueryFolderItem> getQueryFolderItems() throws WvcmException;

    /**
     * A boolean indicating whether or not the effective access rights on this
     * folder allow it to be read by the current user.
     */
    PropertyName<Boolean> IS_READABLE =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-readable");

    /**
     * Returns the value of the {@link #IS_READABLE IS_READABLE} property as
     * defined by this proxy.
     * 
     * @return A boolean indicating whether or not the effective access rights
     *         on this folder allow it to be read by the current user
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_READABLE IS_READABLE} property.
     */
    boolean getIsReadable() throws WvcmException;

    /**
     * A boolean indicating whether or not the effective access rights on this
     * folder allow it to be written by the current user
     */
    PropertyName<Boolean> IS_WRITABLE =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "is-writable");

    /**
     * Returns the value of the {@link #IS_WRITABLE IS_WRITABLE} property as
     * defined by this proxy.
     * 
     * @return A boolean indicating whether or not the effective access rights
     *         on this folder allow it to be read by the current user.
     * 
     * @throws WvcmException if this proxy does not define a value for the
     *             {@link #IS_WRITABLE IS_WRITABLE} property.
     */
    boolean getIsWritable() throws WvcmException;

    /**
     * A PropertyName class for requesting the applied or effective access
     * rights for a specific group on this folder and, optionally, its nested
     * folders
     */
    public class GroupAccessRightsName extends
                    PropertyName<List<StpAccessControlEntry>>
    {
        /**
         * Constructs a PropertyName for requesting the applied or effective
         * access rights for a specific group on this folder and, optionally,
         * its nested folders. The value of this property is a List of
         * StpAccessControlEntry objects.
         * 
         * @param groupName The name of the group for which access rights are to
         *            be returned.
         * @param type The EntryType (APPLIED or EFFECTIVE) of the access rights
         *            to be returned.
         * @param nested <b>true</b> if the access rights of the specified type
         *            and group are also to be returned for the folders nested
         *            within this folder; <b>false</b> if just the access
         *            rights on this folder are to be included.
         */
        public GroupAccessRightsName(String groupName,
                                     EntryType type,
                                     boolean nested)
        {
            super(StpExBase.GROUP_ACCESS_RIGHTS_NAMESPACE, (nested? "nested-"
                                                                  : "")
                + type + "-" + groupName + "-access-rights");
        }
    }

    /**
     * Retrieves from this proxy, the StpAccessControlEntry list returned as the
     * value of a requested property named by a GroupAccessRightsName having the
     * same parameters as this method.
     * 
     * @param groupName The name of the group for which access rights were
     *            requested.
     * @param type The EntryType of the access rights requested.
     * @param nested <b>true</b> if the access rights of the specified type and
     *            group were also to be returned for the folders nested within
     *            this folder; <b>false</b> if just the access rights on this
     *            folder were to be included.
     * @return A list of StpAccessControlEntry objects, each representing an
     *         AccessRight of the group as requested.
     * @throws WvcmException If the implied GroupAccessRights property was not
     *             requested or not otherwise available.
     */
    List<StpAccessControlEntry> getGroupAccessRights(String groupName,
                                                     EntryType type,
                                                     boolean nested)
        throws WvcmException;

    /**
     * A PropertyName class for requesting the applied or effective access
     * rights for a specific user on this folder and, optionally, its nested
     * folders
     */
    public class UserAccessRightsName extends
                    PropertyName<List<StpAccessControlEntry>>
    {
        /**
         * Constructs a PropertyName for requesting the applied or effective
         * access rights for a specific user on this folder and, optionally, its
         * nested folders. The value of this property is a List of
         * StpAccessControlEntry objects.
         * 
         * @param userName The login_name of the user for which access rights
         *            are to be returned.
         * @param type The EntryType (APPLIED or EFFECTIVE) of the access rights
         *            to be returned.
         * @param nested <b>true</b> if the access rights of the specified type
         *            and user are also to be returned for the folders nested
         *            within this folder; <b>false</b> if just the access
         *            rights on this folder are to be included.
         */
        public UserAccessRightsName(String userName,
                                    EntryType type,
                                    boolean nested)
        {
            super(StpExBase.USER_ACCESS_RIGHTS_NAMESPACE, (nested? "nested-"
                                                                 : "")
                + type + "-" + userName + "-access-rights");
        }
    }

    /**
     * Retrieves from this proxy, the StpAccessControlEntry list returned as the
     * value of a requested property named by a UserAccessRightsName having the
     * same parameters as this method.
     * 
     * @param userName The name of the user for which access rights were
     *            requested.
     * @param type The EntryType of the access rights requested.
     * @param nested <b>true</b> if the access rights of the specified type and
     *            user were also to be returned for the folders nested within
     *            this folder; <b>false</b> if just the access rights on this
     *            folder were to be included.
     * @return A list of StpAccessControlEntry objects, each representing an
     *         AccessRight of the type and user as requested.
     * @throws WvcmException If the implied GroupAccessRights property was not
     *             requested or not otherwise available.
     */
    List<StpAccessControlEntry> getUserAccessRights(String userName,
                                                    EntryType type,
                                                    boolean nested)
        throws WvcmException;
    
    /**
     * A special property used to refresh a folder so that children added after
     * this session started can be found. 
     */
    PropertyName<Boolean> NEEDS_REFRESH =
        new PropertyName<Boolean>(PROPERTY_NAMESPACE, "folder-needs-refresh");

    /**
     * Sets the NEEDS_REFRESH property. Doing this will cause the folder's 
     * list of children to be reread. 
     */
    void setNeedsRefresh();

}
